
# q-expansion of newform 4450.2.a.c, downloaded from the LMFDB on 20 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4450
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [0], [4], [-6], [-2], [-3], [5], [3], [0], [5], [10], [0], [1], [-12], [-9], [12], [-10], [4], [-6], [1], [-10], [12], [-1], [-17], [0], [-5], [-18], [-7], [6], [7], [0], [-18], [8], [-12], [-16], [-14], [1], [12], [-3], [18], [-10], [-3], [22], [0], [-10], [-4], [-8], [-24], [-4], [-6], [15], [-22], [-12], [3], [0], [15], [-22], [1], [-6], [-14], [18], [-2], [12], [-20], [27], [-34], [-8], [-18], [-16], [-6], [-24], [16], [25], [-1], [-15], [36], [-20], [-18], [14], [9], [8], [-15], [16], [-1], [-6], [39], [10], [18], [-20], [-18], [-6], [4], [-15], [-4], [-24], [-33], [0], [4], [38], [28], [24], [21], [-18], [47], [-44], [18], [0], [-3], [35], [-2], [-41], [36], [44], [-40], [9], [-32], [-24], [-48], [24], [2], [-29], [30], [-15], [-10], [-15], [-22], [21], [1], [-5], [-7], [3], [-34], [1], [-33], [5], [0], [1], [18], [15], [-16], [30], [-8], [-15], [20], [15], [-56], [-12], [-52], [-33], [4], [6], [19], [-57], [-38], [-48], [29], [15], [7], [36], [48], [36], [1], [0], [9], [-24], [-19], [-26], [-10], [54], [-12], [-31], [24], [28], [8], [30], [26], [-48], [-2], [35], [16], [57], [-23], [-24], [36], [0], [-2], [-2], [32], [-21], [22], [-27], [32], [-6], [48], [-27], [-58], [40], [30], [-39], [-3], [35], [13], [-40], [45], [-12], [-13], [-24], [-24], [44], [28], [69], [34], [9], [-60], [50], [-14], [-30], [-24], [36], [38], [32], [24], [34], [-60], [47], [6], [-24], [25], [-60], [70], [11], [-58], [15], [49], [63], [2], [45], [-48], [-30], [12], [50], [-8], [77], [57], [-48], [-11], [-21], [-4], [-15], [-59], [-30], [42], [-4], [-27], [18], [2], [-62], [-42], [34], [13], [-36], [-10], [61], [42], [44], [3], [30], [22], [-81], [-1], [28], [-44], [-4], [-14], [7], [-9], [35], [-7], [-48], [-69], [-40], [48], [-67], [22], [6], [-50], [18], [-22], [21], [39], [-63], [-66], [-3], [-17], [-39], [56], [-84], [-60], [37], [28], [57], [-43], [-6], [-52], [52], [-24], [50], [-12], [22], [48], [-3], [21], [-2], [66], [65], [15], [-54], [10], [6], [-70], [-17], [-30], [43], [-66], [-13], [65], [-80], [6], [24], [2], [30], [-40], [-18], [47], [48], [77], [-18], [-76], [-29], [82], [63], [21], [8], [-6], [-30], [-70], [28], [39], [54], [-82], [-92], [6], [10], [68], [30], [-12], [24], [-36], [66], [-41], [42], [-30], [-27], [52], [-44], [-54], [16], [92], [12], [80], [33], [-54], [8], [-14], [48], [-66], [64], [-54], [67], [3], [12], [67], [48], [-10], [96], [-100], [82], [-41], [6], [-28], [-36], [63], [91], [45], [-62], [-4], [42], [-49], [57], [-49], [-90], [94], [-3], [-6], [20], [-44], [-54], [70], [-45], [82], [-33], [-6], [77], [-26], [-72], [81], [-56], [-75], [-6], [0], [-74], [-90], [-36], [-11], [30], [15], [-75], [38], [-45]]
